"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.memo = void 0;
const memoize_resolver_1 = require("memoize-resolver");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
function memo(func, memoize) {
    const resolver = memoize_resolver_1.createResolver();
    return (memoize || defaultMemoize)((...args) => func(...args).pipe(operators_1.multicast(new rxjs_1.ReplaySubject(Infinity)), operators_1.refCount()), resolver);
}
exports.memo = memo;
function defaultMemoize(func, resolver) {
    const map = new Map();
    return (...args) => {
        const key = resolver(...args);
        if (map.has(key)) {
            return map.get(key);
        }
        const result = func(...args);
        map.set(key, result);
        return result;
    };
}
//# sourceMappingURL=memo.js.map