"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.debounceSync = void 0;
const rxjs_1 = require("rxjs");
function debounceSync() {
    return (source) => new rxjs_1.Observable((observer) => {
        let actionSubscription;
        let actionValue;
        const rootSubscription = new rxjs_1.Subscription();
        rootSubscription.add(source.subscribe({
            complete: () => {
                if (actionSubscription) {
                    observer.next(actionValue);
                }
                observer.complete();
            },
            error: (error) => observer.error(error),
            next: (value) => {
                actionValue = value;
                if (!actionSubscription) {
                    actionSubscription = rxjs_1.asapScheduler.schedule(() => {
                        observer.next(actionValue);
                        actionSubscription = undefined;
                    });
                    rootSubscription.add(actionSubscription);
                }
            },
        }));
        return rootSubscription;
    });
}
exports.debounceSync = debounceSync;
//# sourceMappingURL=debounceSync.js.map