"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.delayUntil = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
function delayUntil(notifier) {
    return (source) => source.pipe(operators_1.publish((published) => {
        const delayed = new rxjs_1.Observable((subscriber) => {
            let buffering = true;
            const buffer = [];
            const subscription = new rxjs_1.Subscription();
            subscription.add(notifier.subscribe(() => {
                buffer.forEach((value) => subscriber.next(value));
                subscriber.complete();
            }, (error) => subscriber.error(error), () => {
                buffering = false;
                buffer.length = 0;
            }));
            subscription.add(() => {
                buffer.length = 0;
            });
            subscription.add(published.subscribe((value) => buffering && buffer.push(value), (error) => subscriber.error(error)));
            return subscription;
        });
        return rxjs_1.concat(delayed, published);
    }));
}
exports.delayUntil = delayUntil;
//# sourceMappingURL=delayUntil.js.map