"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.rateLimit = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
function rateLimit(period, ...args) {
    let count = 1;
    let scheduler = rxjs_1.asapScheduler;
    if (args.length === 1) {
        if (typeof args[0] === "number") {
            count = args[0];
        }
        else {
            scheduler = args[0];
        }
    }
    else if (args.length === 2) {
        count = args[0];
        scheduler = args[1];
    }
    const definedCount = count || 1;
    return (source) => source.pipe(operators_1.scan((emissions, value) => {
        const now = scheduler.now();
        const since = now - period;
        emissions = emissions.filter((emission) => emission.until > since);
        if (emissions.length >= definedCount) {
            const leastRecentEmission = emissions[0];
            const mostRecentEmission = emissions[emissions.length - 1];
            const until = leastRecentEmission.until +
                period * Math.floor(emissions.length / definedCount);
            emissions.push({
                delay: mostRecentEmission.until < now
                    ? until - now
                    : until - mostRecentEmission.until,
                until,
                value,
            });
        }
        else {
            emissions.push({
                delay: 0,
                until: now,
                value,
            });
        }
        return emissions;
    }, []), operators_1.map((emissions) => emissions[emissions.length - 1]), operators_1.concatMap((emission) => {
        const observable = rxjs_1.of(emission.value);
        return emission.delay
            ? observable.pipe(operators_1.delay(emission.delay, scheduler))
            : observable;
    }));
}
exports.rateLimit = rateLimit;
//# sourceMappingURL=rateLimit.js.map