"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.refCountAuditTime = exports.refCountDelay = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
function refCountDelay(duration, scheduler = rxjs_1.asapScheduler) {
    return (source) => {
        const connectable = source;
        let connectableSubscription = null;
        let connectorSubscription = null;
        const notifier = new rxjs_1.Subject();
        const connector = notifier.pipe(operators_1.scan((count, step) => count + step, 0), operators_1.switchMap((count) => {
            if (count === 0) {
                return rxjs_1.timer(duration, scheduler).pipe(operators_1.tap(() => {
                    if (connectableSubscription) {
                        connectableSubscription.unsubscribe();
                        connectableSubscription = null;
                    }
                    if (connectorSubscription) {
                        connectorSubscription.unsubscribe();
                        connectorSubscription = null;
                    }
                }));
            }
            if (!connectableSubscription && count > 0) {
                return rxjs_1.timer(0, scheduler).pipe(operators_1.tap(() => {
                    if (!connectableSubscription) {
                        connectableSubscription = connectable.connect();
                    }
                }));
            }
            return rxjs_1.NEVER;
        }));
        return rxjs_1.using(() => {
            if (!connectorSubscription) {
                connectorSubscription = connector.subscribe();
            }
            notifier.next(1);
            return { unsubscribe: () => notifier.next(-1) };
        }, () => source);
    };
}
exports.refCountDelay = refCountDelay;
exports.refCountAuditTime = refCountDelay;
//# sourceMappingURL=refCountDelay.js.map