"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tapSubscribe = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
function tapSubscribe(configOrSubscribe) {
    const { ignore = {}, subscribe = rxjs_1.noop, unsubscribe = rxjs_1.noop } = typeof configOrSubscribe === "function"
        ? { subscribe: configOrSubscribe }
        : configOrSubscribe;
    return (source) => rxjs_1.defer(() => {
        let completed = false;
        let errored = false;
        subscribe();
        return source.pipe(operators_1.tap({
            complete: () => (completed = true),
            error: () => (errored = true),
        }), operators_1.finalize(() => {
            if (completed && ignore.complete) {
                return;
            }
            if (errored && ignore.error) {
                return;
            }
            unsubscribe();
        }));
    });
}
exports.tapSubscribe = tapSubscribe;
//# sourceMappingURL=tapSubscribe.js.map