import { Observable, Subscription } from "rxjs";
export function mergeHigherOrderArray() {
    return (higherOrder) => new Observable((observer) => {
        let lasts = [];
        let nexts = [];
        let higherOrderCompleted = false;
        const higherOrderSubscription = new Subscription();
        higherOrderSubscription.add(higherOrder.subscribe((observables) => {
            const subscribes = [];
            nexts = observables.map((observable) => {
                const index = lasts.findIndex((last) => last.observable === observable);
                if (index !== -1) {
                    const next = lasts[index];
                    lasts.splice(index, 1);
                    return next;
                }
                const next = { completed: false, observable };
                subscribes.push(() => {
                    if (higherOrderSubscription.closed) {
                        return;
                    }
                    next.subscription = next.observable.subscribe((value) => observer.next(value), (error) => observer.error(error), () => {
                        next.completed = true;
                        if (higherOrderCompleted &&
                            nexts.every(({ completed }) => completed)) {
                            observer.complete();
                        }
                    });
                    higherOrderSubscription.add(next.subscription);
                });
                return next;
            });
            lasts.forEach(({ subscription }) => {
                if (subscription) {
                    subscription.unsubscribe();
                }
            });
            lasts = nexts;
            subscribes.forEach((subscribe) => subscribe());
        }, (error) => observer.error(error), () => {
            if (lasts.every(({ completed }) => completed)) {
                observer.complete();
            }
            higherOrderCompleted = true;
        }));
        return higherOrderSubscription;
    });
}
//# sourceMappingURL=mergeHigherOrderArray.js.map