import { Subject } from "rxjs";
import { startWith, switchMap } from "rxjs/operators";
export function resettable(factory, ...args) {
    const resetter = new Subject();
    const source = new Subject();
    let destination = factory(...args);
    let subscription = source.subscribe(destination);
    return {
        reset(...args) {
            subscription.unsubscribe();
            destination = factory(...args);
            subscription = source.subscribe(destination);
            resetter.next();
        },
        unsubscribe() {
            subscription.unsubscribe();
            source.unsubscribe();
        },
        get closed() {
            return subscription.closed;
        },
        get observable() {
            return resetter.asObservable().pipe(startWith(null), switchMap(() => destination));
        },
        get subject() {
            return source;
        },
    };
}
//# sourceMappingURL=resettable.js.map