import { concat, from, identity, Observable, of, Subject, } from "rxjs";
import { expand, ignoreElements, mergeMap, tap } from "rxjs/operators";
import { NotificationQueue } from "./NotificationQueue";
export function traverse({ concurrency = 1, factory, operator = identity, notifier, }) {
    return new Observable((observer) => {
        let queue;
        let queueOperator;
        if (notifier) {
            queue = new NotificationQueue(notifier);
            queueOperator = identity;
        }
        else {
            const subject = new Subject();
            queue = new NotificationQueue(subject);
            queueOperator = (markers) => {
                subject.next();
                return markers;
            };
        }
        const destination = new Subject();
        const subscription = destination.subscribe(observer);
        subscription.add(queue.connect());
        subscription.add(of(undefined)
            .pipe(expand((marker) => queue.pipe(mergeMap((index) => factory(marker, index).pipe(mergeMap(({ markers, values }) => concat(from(values).pipe(operator, tap((value) => destination.next(value)), ignoreElements()), from(markers))))), queueOperator), concurrency))
            .subscribe({
            complete: () => destination.complete(),
            error: (error) => destination.error(error),
        }));
        return subscription;
    });
}
//# sourceMappingURL=traverse.js.map